# Installed as usr/share/bug/@PACKAGE@/presubj in @PACKAGE@
# See dh_bugfiles(1) and /usr/share/doc/reportbug/README.developers.gz
# The contents of this file are displayed to the user by the bug reporting tool.  
