# Installed as usr/share/bug/@PACKAGE@ or usr/share/bug/@PACKAGE@/script in @PACKAGE@
# See dh_bugfiles(1) and /usr/share/doc/reportbug/README.developers.gz
# This is the script to be run by the bug reporting program for generating a bug report template.  
