= @UPACKAGE@(1)
:man source:   @UCPACKAGE@
:man version:  @VERREV@
:man manual:   @UCPACKAGE@ Manual

// See "Manpage Documents" in "AsciiDoc User Guide"
// http://www.methods.co.nz/asciidoc/userguide.html

// convert by:
// $ a2x --doctype manpage --format manpage @PACKAGE@.asciidoc

== NAME

@PACKAGE@ - program to do something

== SYNOPSIS

*@PACKAGE@*  [*-h*] ... 


== DESCRIPTION

*@PACKAGE@* do something.

=== optional arguments:

*-h*, *--help*::
    show this help message and exit.


== AUTHOR

Copyright © @YEAR@ @FULLNAME@ <@EMAIL@>

== LICENSE

MIT License

== SEE ALSO

See also ...

