# Defaults for @PACKAGE@ initscript
# sourced by /etc/init.d/@PACKAGE@
# installed at /etc/default/@PACKAGE@ by the maintainer scripts

#
# This is a POSIX shell fragment
#

# Additional options that are passed to the Daemon.
DAEMON_OPTS=""
