# See dh_installdeb(1).
# Files listed in this file are treated as user configuration files by the
# Debian system and will never be overwritten without user consent.
#
# In v3 compatibility mode and higher, all files in the etc/ directory in
# a package will automatically be flagged as conffiles by this program, so
# there is no need to list them manually here.
#
# For most package types, the only place conffiles should ever be is under
# /etc, and so this file doesn’t need to exist.
