# Installed as usr/share/bug/@PACKAGE@/control in @PACKAGE@
# See dh_bugfiles(1) and /usr/share/doc/reportbug/README.developers.gz
# It is the bug control file containing some directions for the bug reporting tool.
