# See dh_installdirs(1)
# Lists directories to be created in package. (w/o preceding slash)
# Do not bother to list directories here if they are already listed in 
# @PACKAGE@.install etc.
# Any directories which we need but which are not created by the normal 
# installation procedure (make install DESTDIR=... invoked by 
# dh_auto_install). This generally means there is a problem with the 
# Makefile.

